{% extends "orga/submission/base.html" %}
{% load i18n %}
{% load bootstrap4 %}

{% block submission_content %}
    <form method="post">
        <div class="ml-auto mr-auto col-md-8">
            <h3>
                {% include "cfp/event/fragment_state.html" with state=submission.state %}
                <i class="fa fa-arrow-right"></i>
                {% include "cfp/event/fragment_state.html" with state=target %}
            </h3>
            <p>
                {% translate "Please confirm that you really want to change the state of this proposal." %}
            </p>

            {% csrf_token %}
            <input type="hidden" name="next" value="{{ next }}">
            {% bootstrap_field form.pending %}
        </div>

        <div class="submit-group" id="submission-state-change"><span></span><span>
            <a
                class="btn btn-lg btn-outline-info"
                href="{% if request.GET.next %}{{ request.GET.next }}{% else %}{{ submission.orga_urls.base }}{% endif %}"
            >
                {% translate "Back" %}
            </a>
            <button type="submit" class="btn btn-lg submission-state-{{ target }}">
                {% translate "Do it" %}
            </button>
        </span></div>
    </form>
{% endblock %}
